import unittest
import pydarm
import numpy as np


class TestOpticalResponse(unittest.TestCase):

    def setUp(self):
        # Pre-computed values from an optical plant with
        # f_cc = 400 Hz, f_s = 1 Hz, Q = 10 (anti-spring)
        # And frequencies = np.logspace(0, np.log10(5000.), 10)
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_optical_response = np.array(
            [0.4988123437694849+0.023690625000925447j,
             0.8682289568018182+0.023690785759931294j,
             0.9775621260186113-0.0018192134833768408j,
             0.9949877935396494-0.03672708739347625j,
             0.9877484188511914-0.10651841758830315j,
             0.9256481649097499-0.26176414634118933j,
             0.6518895845322998-0.47619218998268714j,
             0.21999081206271484-0.41417325269496474j,
             0.04075815123481037-0.19770363308768762j,
             0.006360890045023226-0.0794911255643983j])
        optical_response_model = [400, 1, 10]
        self.optical_response = pydarm.sensing.SensingModel.optical_response(
            optical_response_model[0], optical_response_model[1],
            optical_response_model[2], pro_spring=False)

    def tearDown(self):
        del self.frequencies
        del self.known_optical_response
        del self.optical_response

    def test_optical_response(self):
        """ Test the optical plant model """
        optical_response_freqresp = pydarm.utils.freqrespZPK(
            self.optical_response, 2.0*np.pi*self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(np.abs(optical_response_freqresp[n]),
                                   np.abs(self.known_optical_response[n]))
            self.assertAlmostEqual(
                np.angle(optical_response_freqresp[n], deg=True),
                np.angle(self.known_optical_response[n], deg=True))


class TestMeanArmLength(unittest.TestCase):

    def setUp(self):
        # computed from
        # x_arm_length = 3994.4704
        # y_arm_length = 3994.4692
        self.known_mean = 3994.4698

    def tearDown(self):
        del self.known_mean

    def test_mean_arm_length(self):
         C = pydarm.sensing.SensingModel('''
[metadata]
[interferometer]
[sensing]
x_arm_length = 3994.4704
y_arm_length = 3994.4692
''')
         test_mean = C.mean_arm_length()
         self.assertAlmostEqual(test_mean, self.known_mean)


class TestOMCDCPDTransimpedenceAmplifierResponse(unittest.TestCase):

    def setUp(self):
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_response = np.array(
            [0.9999999874151851-0.0001291724749149135j,
             0.9999999164704483-0.00033278721023225985j,
             0.9999994455870466-0.0008573598787775811j,
             0.9999963201854518-0.002208812789960272j,
             0.9999755761485756-0.005690487871025065j,
             0.9998379035432662-0.014659051083653684j,
             0.9989246706803299-0.03774287852774699j,
             0.9928871918753837-0.09684080299079524j,
             0.9538456946366823-0.24287440441692343j,
             0.7340692184563671-0.5282641958643781j])

    def tearDown(self):
        del self.frequencies
        del self.known_response

    def test_omc_dcpd_transimpedence_amplifier_response(self):
        C = pydarm.sensing.SensingModel('''
[metadata]
[interferometer]
[sensing]
omc_path_names = A
omc_meas_p_trans_amplifier_uncompensated = 13.7e3, 17.8e3
omc_front_end_trans_amplifier_compensation = ON
''')
        test_response = C.omc_dcpd_transimpedence_amplifier_response(
            'A', self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(test_response[n]),
                np.abs(self.known_response[n]))
            self.assertAlmostEqual(
                np.angle(test_response[n], deg=True),
                np.angle(self.known_response[n], deg=True))


class TestOMCDCPDWhiteningResponse(unittest.TestCase):

    def setUp(self):
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_response = np.array(
            [0.9999999840941629-0.00014897328868210543j,
             0.9999998944277334-0.00038380006810453216j,
             0.9999992992823787-0.000988784271267042j,
             0.999995349116734-0.002547399857003313j,
             0.9999691309991117-0.006562759807444474j,
             0.9997951317032564-0.016905766838847664j,
             0.9986410763986874-0.0435222188297893j,
             0.9910178637120639-0.11157937182260591j,
             0.9419873353388445-0.2783804818601435j,
             0.6742077533753816-0.5876961400786618j])

    def tearDown(self):
        del self.frequencies
        del self.known_response

    def test_omc_dcpd_whitening_response(self):
        C = pydarm.sensing.SensingModel('''
[metadata]
[interferometer]
[sensing]
omc_path_names = A
whitening_mode_names = test
omc_meas_p_whitening_uncompensated_test = 11.346e3, 32.875e3, 32.875e3
omc_front_end_whitening_compensation_test = ON
''')
        test_response = C.omc_dcpd_whitening_response(
            'A', self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(test_response[n]),
                np.abs(self.known_response[n]))
            self.assertAlmostEqual(
                np.angle(test_response[n], deg=True),
                np.angle(self.known_response[n], deg=True))


class TestOMCAnalaogDCPDReadoutResponse(unittest.TestCase):

    def setUp(self):
        # Pre-computed values
        # omc_meas_p_trans_amplifier_A   = 13.7e3, 17.8e3
        # omc_meas_p_trans_whitening_A = \
        #     11.346e3, 32.875e3, 32.875e3
        # analog_anti_aliasing_file_A = Common/pyDARM/H1aa.mat
        # frequencies = np.logspace(0, np.log10(5000.), 10)
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_analog_response = np.array(
            [0.9999998536665746-0.0005167286242298383j,
             0.9999990287350008-0.00133124823161614j,
             0.9999935534012022-0.003429689154918376j,
             0.9999572121767486-0.008835784518168556j,
             0.9997160196619771-0.02276144831389675j,
             0.998115856554862-0.0586028191391922j,
             0.9875263088271709-0.15033813445138738j,
             0.9186003414470439-0.3765222040547237j,
             0.5164527749967376-0.8019627240716991j,
             -0.617879650294782-0.4291979256047118j])

    def tearDown(self):
        del self.frequencies
        del self.known_analog_response

    def test_omc_analog_dcpd_readout_response(self):
        """ Test the uncompensated OMC DCPD poles response """
        C = pydarm.sensing.SensingModel('''
[metadata]
[interferometer]
[sensing]
omc_path_names = A
omc_meas_p_trans_amplifier_uncompensated = 13.7e3, 17.8e3
whitening_mode_names = test
omc_meas_p_whitening_uncompensated_test = 11.346e3, 32.875e3, 32.875e3
super_high_frequency_poles_apparent_delay = 0
analog_anti_aliasing_file = test/H1aa.mat
omc_front_end_trans_amplifier_compensation = ON
omc_front_end_whitening_compensation_test = ON
''')
        analog_dcpd_response = C.omc_analog_dcpd_readout_response('A',
            self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(analog_dcpd_response[n]),
                np.abs(self.known_analog_response[n]))
            self.assertAlmostEqual(
                np.angle(analog_dcpd_response[n], deg=True),
                np.angle(self.known_analog_response[n], deg=True))


class TestOMCDigitalFiltersResponse(unittest.TestCase):

    def setUp(self):
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_digital_response = np.array(
            [0.00031477507457620236-0.00041932541064786685j,
             6.351752486888009e-06-0.0002512689712022434j,
             -3.998380486351784e-05-6.492346501958175e-05j,
             -1.0702299095113614e-05-1.0070777528904742e-05j,
             -1.2659064264492532e-06-2.4689933152042374e-06j,
             3.0955872674940336e-07-8.70312552324717e-07j,
             5.502073347929391e-07-3.3236246245694016e-07j,
             5.865370982993107e-07-1.2794394638990852e-07j,
             5.920112453308542e-07-4.7663182537616464e-08j,
             5.928278462061573e-07-1.3059142838018548e-08j])

    def tearDown(self):
        del self.frequencies
        del self.known_digital_response

    def test_omc_digital_filters_response(self):
        C = pydarm.sensing.SensingModel('''
[metadata]
[interferometer]
[sensing]
omc_path_names = A
omc_filter_file = test/H1OMC_1239468752.txt
omc_filter_bank = OMC_DCPD_A
omc_filter_noncompensating_modules = 1,4,5,7
omc_filter_gain = 1000
whitening_mode_names = test
omc_front_end_trans_amplifier_compensation = ON
omc_front_end_whitening_compensation_test = ON
''')
        digital_filters_response = C.omc_digital_filters_response('A',
            self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(digital_filters_response[n]),
                np.abs(self.known_digital_response[n]))
            self.assertAlmostEqual(
                np.angle(digital_filters_response[n], deg=True),
                np.angle(self.known_digital_response[n], deg=True))


class TestOmcPathResponse(unittest.TestCase):

    def setUp(self):
        # Pre-computed values
        # omc_meas_p_trans_amplifier_A   = 13.7e3, 17.8e3
        # omc_meas_p_trans_whitening_A = \
        #     11.346e3, 32.875e3, 32.875e3
        # gain_ratio_A = 1
        # balance_matrix_A = 1
        # anti_aliasing_rate_string = 16k
        # anti_aliasing_method      = biquad
        # analog_anti_aliasing_file_A = test/H1aa.mat
        # frequencies = np.logspace(0, np.log10(5000.), 10)
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_response = np.array(
            [1.0000003891873206-0.0007853244803941936j,
             0.9999986653249536-0.0020232314749344927j,
             0.9999872235010034-0.005212435716577928j,
             0.9999112811210966-0.013428491813741426j,
             0.9994072604116898-0.03459048056077152j,
             0.996063400254325-0.08902383023813454j,
             0.9739348784621216-0.22778552216162168j,
             0.8299851258734532-0.5601596601010437j,
             0.010770373018644919-1.0030495286483354j,
             -0.2267642153914641+0.7704794442924117j])

    def tearDown(self):
        del self.frequencies
        del self.known_response

    def test_omc_path_response(self):
        """ Test the OMC DCPD path response """
        C = pydarm.sensing.SensingModel('''
[metadata]
[interferometer]
[sensing]
omc_path_names = A
omc_meas_z_trans_amplifier_uncompensated =
omc_meas_p_trans_amplifier_uncompensated = 13.7e3, 17.8e3
whitening_mode_names = test
omc_meas_z_whitening_uncompensated_test =
omc_meas_p_whitening_uncompensated_test = 11.346e3, 32.875e3, 32.875e3
super_high_frequency_poles_apparent_delay = 0
analog_anti_aliasing_file = test/H1aa.mat
anti_aliasing_rate_string = 16k
anti_aliasing_method      = biquad
gain_ratio = 1
balance_matrix = 1
adc_gain = 1638.001638001638
omc_filter_file = test/H1OMC_1239468752.txt
omc_filter_bank = OMC_DCPD_A
omc_filter_noncompensating_modules = 4
omc_filter_gain = 1
omc_front_end_trans_amplifier_compensation = ON
omc_front_end_whitening_compensation_test = ON
''')
        omc_dcpd_response = C.omc_path_response('A', self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(omc_dcpd_response[n]),
                np.abs(self.known_response[n]))
            self.assertAlmostEqual(
                np.angle(omc_dcpd_response[n], deg=True),
                np.angle(self.known_response[n], deg=True))


class TestCombinePathResponses(unittest.TestCase):

    def setUp(self):
        # Pre-computed values
        # omc_meas_p_trans_amplifier_A   = 13.7e3, 17.8e3
        # omc_meas_p_trans_amplifier_B   = 13.7e3, 17.8e3
        # omc_meas_p_trans_whitening_A = \
        #     11.346e3, 32.875e3, 32.875e3
        # omc_meas_p_trans_whitening_B = \
        #     11.521e3, 32.863e3, 32.863e3
        # gain_ratio_A = 1
        # gain_ratio_B = 1
        # balance_matrix_A = 1
        # balance_matrix_B = 1
        # anti_aliasing_rate_string = 16k
        # anti_aliasing_method      = biquad
        # analog_anti_aliasing_file_A = test/H1aa.mat
        # analog_anti_aliasing_file_B = test/H1aa.mat
        # frequencies = np.logspace(0, np.log10(5000.), 10)
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_response = np.array(
            [1.0000003897620588-0.0007846662038375172j,
             0.9999986691396795-0.002021535559784837j,
             0.9999872488204784-0.00520806659053789j,
             0.9999114491696466-0.013417236697166764j,
             0.9994083755791304-0.03456150226440034j,
             0.9960707920317725-0.0889494857211583j,
             0.9739835023107202-0.2275993050582223j,
             0.8302887998715469-0.5597681458332082j,
             0.012018214381908476-1.003248388113367j,
             -0.22916763899233059+0.7707887449719427j])

    def tearDown(self):
        del self.frequencies
        del self.known_response

    def test_omc_combine_path_responses(self):
        """ Test the combined OMC DCPD path response """
        C = pydarm.sensing.SensingModel('''
[metadata]
[interferometer]
[sensing]
omc_path_names = A, B
anti_aliasing_rate_string = 16k
anti_aliasing_method      = biquad
analog_anti_aliasing_file = test/H1aa.mat, test/H1aa.mat
omc_meas_p_trans_amplifier_uncompensated   = 13.7e3, 17.8e3: 13.7e3, 17.8e3
whitening_mode_names = test, test
omc_meas_p_whitening_uncompensated_test   = 11.346e3, 32.875e3, 32.875e3: 11.521e3, 32.863e3, 32.863e3
super_high_frequency_poles_apparent_delay = 0, 0
gain_ratio = 1, 1
balance_matrix = 1, 1
adc_gain = 1638.001638001638, 1638.001638001638
omc_filter_file = test/H1OMC_1239468752.txt
omc_filter_bank = OMC_DCPD_A, OMC_DCPD_B
omc_filter_noncompensating_modules = 4: 4
omc_filter_gain = 1, 1
omc_front_end_trans_amplifier_compensation = ON, ON
omc_front_end_whitening_compensation_test = ON, ON
''')
        omc_dcpd_response = C.omc_combine_path_responses(self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(omc_dcpd_response[n]),
                np.abs(self.known_response[n]))
            self.assertAlmostEqual(
                np.angle(omc_dcpd_response[n], deg=True),
                np.angle(self.known_response[n], deg=True))


class TestLightTravelTimeDelayResponse(unittest.TestCase):

    def setUp(self):
        # Pre-computed values
        # x_arm_length = 3994.4704
        # y_arm_length = 3994.4692
        # frequencies = np.logspace(0, np.log10(5000.), 10)
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_light_travel_time_delay_response = np.array(
            [0.999999996495657-8.371789635887121e-05j,
             0.9999999767405227-0.00021568253099713038j,
             0.9999998456192037-0.0005556631792446989j,
             0.9999989753240384-0.0014315554035123075j,
             0.9999931988952449-0.003688111068729696j,
             0.9999548591197768-0.009501564226344633j,
             0.9997003978414927-0.024476816695829077j,
             0.9980120016904314-0.0630241579226448j,
             0.9868296389458847-0.16176298618019783j,
             0.9136631851422247-0.40647211972749725j])

    def tearDown(self):
        del self.frequencies
        del self.known_light_travel_time_delay_response

    def test_light_travel_time_delay_response(self):
        """ Test the light travel time delay response """
        C = pydarm.sensing.SensingModel('''
[metadata]
[interferometer]
[sensing]
x_arm_length = 3994.4704
y_arm_length = 3994.4692
''')
        light_travel_time_response = \
            C.light_travel_time_delay_response(self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(light_travel_time_response[n]),
                np.abs(self.known_light_travel_time_delay_response[n]))
            self.assertAlmostEqual(
                np.angle(light_travel_time_response[n], deg=True),
                np.angle(self.known_light_travel_time_delay_response[n],
                         deg=True))


class TestSinglePoleCorrectionResponse(unittest.TestCase):

    def setUp(self):
        # Pre-computed values
        # single_pole_approximation_delay_correction = -12e-6
        # frequencies = np.logspace(0, np.log10(5000.), 10)
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_single_pole_correction_response = np.array(
            [0.9999999971575539+7.539822361471657e-05j,
             0.9999999811337508+0.00019424854695853638j,
             0.9999998747785026+0.000500442783004694j,
             0.999999168863843+0.0012892911320624648j,
             0.9999944834803304+0.0033215973427495236j,
             0.9999633852347264+0.008557347130170182j,
             0.9997569841231653+0.022044788430672032j,
             0.9983873919841177+0.05676808546315201j,
             0.9893128028198163+0.14580870405020152j,
             0.9297764858882515+0.36812455268467786j])

    def tearDown(self):
        del self.frequencies
        del self.known_single_pole_correction_response

    def test_single_pole_correction_response(self):
        """ Test the single pole approximation correction response """
        C = pydarm.sensing.SensingModel('''
[metadata]
[interferometer]
[sensing]
single_pole_approximation_delay_correction = -12e-6
''')
        single_pole_correction_response = \
            C.single_pole_approximation_delay_correction_response(
                self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(single_pole_correction_response[n]),
                np.abs(self.known_single_pole_correction_response[n]))
            self.assertAlmostEqual(
                np.angle(single_pole_correction_response[n], deg=True),
                np.angle(self.known_single_pole_correction_response[n],
                         deg=True))


class TestSensingResidual(unittest.TestCase):

    def setUp(self):
        # Pre-computed values
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_residual = np.array(
            [1.0000003831992847-0.0007929858798234665j,
             0.9999986255804106-0.002042969515282433j,
             0.9999869597043107-0.0052632862823899385j,
             0.9999095302557972-0.013559488366496989j,
             0.9993956411093269-0.03492779907418393j,
             0.995986356934377-0.08988999137528689j,
             0.9734269429695694-0.22996802633705743j,
             0.8267642616923517-0.5649608162253222j,
             -0.004181642663901519-1.003311656263212j,
             -0.19691508479464048+0.7796521952236531j])

    def tearDown(self):
        del self.frequencies
        del self.known_residual

    def test_sensing_residual(self):
        """ Test the computation of the sensing function """
        C = pydarm.sensing.SensingModel('''
[metadata]
[interferometer]
[sensing]
x_arm_length = 3994.4704
y_arm_length = 3994.4692
sensing_sign = 1
anti_aliasing_rate_string = 16k
anti_aliasing_method      = biquad
analog_anti_aliasing_file = test/H1aa.mat, test/H1aa.mat
omc_meas_p_trans_amplifier_uncompensated   = 13.7e3, 17.8e3: 13.7e3, 17.8e3
whitening_mode_names = test, test
omc_meas_p_whitening_uncompensated_test   = 11.346e3, 32.875e3, 32.875e3: 11.521e3, 32.863e3, 32.863e3
super_high_frequency_poles_apparent_delay = 0, 0
gain_ratio = 1, 1
balance_matrix = 1, 1
omc_path_names = A, B
single_pole_approximation_delay_correction = -12e-6
adc_gain = 1638.001638001638, 1638.001638001638
omc_filter_file = test/H1OMC_1239468752.txt
omc_filter_bank = OMC_DCPD_A, OMC_DCPD_B
omc_filter_noncompensating_modules = 4: 4
omc_filter_gain = 1, 1
omc_front_end_trans_amplifier_compensation = ON, ON
omc_front_end_whitening_compensation_test = ON, ON
''')
        C_res = C.sensing_residual(self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(C_res[n]) / np.abs(self.known_residual[n]), 1.0)
            self.assertAlmostEqual(
                np.angle(C_res[n], deg=True) -
                np.angle(self.known_residual[n], deg=True), 0.0)


class TestComputeSensing(unittest.TestCase):

    def setUp(self):
        # Pre-computed values
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_sensing = np.array(
            [-169801.8807036616-219.12048303424578j,
             -1603528.6580281004-13226.702333005833j,
             5879871.803166859-264958.70154167747j,
             3446578.5609586663-208980.6577969556j,
             3201530.8874004283-463525.30679096654j,
             2908780.7008725638-1096359.5702610482j,
             1729667.5321780506-1973342.1148503236j,
             -154875.32218105765-1535395.702427836j,
             -654801.7126515587-135654.7883568197j,
             200533.6778938555+68533.88088596147j])

    def tearDown(self):
        del self.frequencies
        del self.known_sensing

    def test_compute_sensing(self):
        """ Test the computation of the sensing function """
        C = pydarm.sensing.SensingModel('''
[metadata]
[interferometer]
[sensing]
x_arm_length = 3994.4704
y_arm_length = 3994.4692
coupled_cavity_optical_gain = 3.22e6
coupled_cavity_pole_frequency = 410.6
detuned_spring_frequency = 4.468
detuned_spring_Q = 52.14
sensing_sign = 1
is_pro_spring = True
anti_aliasing_rate_string = 16k
anti_aliasing_method      = biquad
analog_anti_aliasing_file = test/H1aa.mat, test/H1aa.mat
omc_meas_p_trans_amplifier_uncompensated   = 13.7e3, 17.8e3: 13.7e3, 17.8e3
whitening_mode_names = test, test
omc_meas_p_whitening_uncompensated_test   = 11.346e3, 32.875e3, 32.875e3: 11.521e3, 32.863e3, 32.863e3
super_high_frequency_poles_apparent_delay = 0, 0
gain_ratio = 1, 1
balance_matrix = 1, 1
omc_path_names = A, B
single_pole_approximation_delay_correction = -12e-6
adc_gain = 1638.001638001638, 1638.001638001638
omc_filter_file = test/H1OMC_1239468752.txt
omc_filter_bank = OMC_DCPD_A, OMC_DCPD_B
omc_filter_noncompensating_modules = 4: 4
omc_filter_gain = 1, 1
omc_front_end_trans_amplifier_compensation = ON, ON
omc_front_end_whitening_compensation_test = ON, ON
''')
        sensing_tf = C.compute_sensing(self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(sensing_tf[n]) / np.abs(self.known_sensing[n]), 1.0)
            self.assertAlmostEqual(
                np.angle(sensing_tf[n], deg=True) -
                np.angle(self.known_sensing[n], deg=True), 0.0)


if __name__ == '__main__':
    unittest.main()



import unittest
import pydarm
import numpy as np
import os
import tempfile


class TestOpticalResponseRatio(unittest.TestCase):

    def setUp(self):
        self.frequencies = np.logspace(0, np.log10(5000), 10)
        self.known_ratio = np.array(
            [0.9687610764499911+0.2047970729401173j,
             0.9924685506670072+0.11021624181361893j,
             0.9996996369445023-0.017419284109037984j,
             1.0000395381056308+9.034598303705724e-05j,
             1.0000460037549426-0.0015728765784223393j,
             0.9999143680349047-0.0053037784890201944j,
             0.9995587616354148-0.013435445411127945j,
             1.0028033347285978-0.03182314724508758j,
             1.037094494083799-0.08469882750680675j,
             1.3450148887458857-0.4063843161408741j])

    def tearDown(self):
        del self.frequencies
        del self.known_ratio

    def test_optical_response_ratio(self):
        calcs = pydarm.calcs.CALCSModel('''
[metadata]
[interferometer]
[sensing]
coupled_cavity_optical_gain = 3.22e6
coupled_cavity_pole_frequency = 410.6
detuned_spring_frequency = 4.468
detuned_spring_Q = 52.14
is_pro_spring = True
[calcs]
foton_invsensing_tf = \
  test/2019-04-04_H1CALCS_InverseSensingFunction_Foton_SRCD-2N_Gain_tf.txt
[actuation]
[digital]
''')
        test_ratio = calcs.optical_response_ratio(self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(test_ratio[n]), np.abs(self.known_ratio[n]))
            self.assertAlmostEqual(
                np.angle(test_ratio[n], deg=True),
                np.angle(self.known_ratio[n], deg=True))


class TestSusResponseRatio(unittest.TestCase):

    def setUp(self):
        self.frequencies = np.logspace(0, np.log10(5000), 10)
        self.known_ratio = np.array(
            [0.8487526667231122-0.023180214028295554j,
             1.000634769947796+0.0006302384282468779j,
             1.0005977801588317+0.000363760972346402j,
             1.000597290467924+0.0005874422057399667j,
             1.000637303467556+0.0013777513334851089j,
             1.0009076909644028+0.003498218128442966j,
             1.0027066353202043+0.009016021154521546j,
             1.0148035382784029+0.023637108614939135j,
             1.10165381013647+0.06878863262289596j,
             2.235309261287587+0.5026819806767127j])

    def tearDown(self):
        del self.frequencies
        del self.known_ratio

    def test_sus_response_ratio(self):
        calcs = pydarm.calcs.CALCSModel('''
[metadata]
[interferometer]
[sensing]
[digital]
[actuation_x_arm]
suspension_file = test/H1susdata_O3.mat
[actuation]
[calcs]
xarm_tst_analog = test/H1CALCS_ETMX_L3_ANALOG.txt
''')
        test_ratio = calcs.sus_response_ratio(self.frequencies, arm='x',
                                              stage='tst')
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(test_ratio[n]), np.abs(self.known_ratio[n]))
            self.assertAlmostEqual(
                np.angle(test_ratio[n], deg=True),
                np.angle(self.known_ratio[n], deg=True))


class TestGdsSensingCorrection(unittest.TestCase):

    def setUp(self):
        # Pre-computed values
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_C_corr = np.array(
            [0.9690020217321972+0.203657344927382j,
             0.9927987606585151+0.10720767543223012j,
             0.9995339485303539-0.025225038758617827j,
             0.9998404602179516-0.020026699699416443j,
             0.9986274137784608-0.053379506643647906j,
             0.9903407804505594-0.13838726788917072j,
             0.9366306937437336-0.3499516632708973j,
             0.6085919368544006-0.7993628642506994j,
             -0.7703535216582147-0.704625047873796j,
             1.0438549765849148-0.4323868901519636j])
        self.known_gds_correction = np.array(
            [0.9883325892921246-0.2077200940000379j,
             0.9956434485655548-0.10751485991908585j,
             0.9998294812828395+0.0252324970597059j,
             0.9997584654376137+0.020025057352554037j,
             0.9985214873445555+0.053373844571182875j,
             0.9904141727779854+0.1383975235141808j,
             0.9368712606521091+0.3500415458577869j,
             0.6029452733652817+0.7919461818617544j,
             -0.7067846031289654+0.6464799871936803j,
             0.8176890307238232+0.3387041543479065j])
        self.model_string = '''
[metadata]
[interferometer]
[sensing]
x_arm_length = 3994.4704
y_arm_length = 3994.4692
coupled_cavity_optical_gain = 3.22e6
coupled_cavity_pole_frequency = 410.6
detuned_spring_frequency = 4.468
detuned_spring_Q = 52.14
sensing_sign = 1
is_pro_spring = True
anti_aliasing_rate_string = 16k
anti_aliasing_method      = biquad
analog_anti_aliasing_file = test/H1aa.mat, test/H1aa.mat
omc_meas_p_trans_amplifier_uncompensated   = 13.7e3, 17.8e3: 13.7e3, 17.8e3
whitening_mode_names = test, test
omc_meas_p_whitening_uncompensated_test   = 11.346e3, 32.875e3, 32.875e3: 11.521e3, 32.863e3, 32.863e3
gain_ratio = 1, 1
balance_matrix = 1, 1
omc_path_names = A, B
single_pole_approximation_delay_correction = -12e-6
super_high_frequency_poles_apparent_delay = 0, 0
adc_gain = 1638.001638001638, 1638.001638001638
omc_filter_file = test/H1OMC_1239468752.txt
omc_filter_bank = OMC_DCPD_A, OMC_DCPD_B
omc_filter_noncompensating_modules = 4: 4
omc_filter_gain = 1, 1
omc_front_end_trans_amplifier_compensation = ON, ON
omc_front_end_whitening_compensation_test = ON, ON
[calcs]
foton_invsensing_tf = \
  test/2019-04-04_H1CALCS_InverseSensingFunction_Foton_SRCD-2N_Gain_tf.txt
[actuation]
[digital]
'''

    def tearDown(self):
        del self.frequencies
        del self.known_gds_correction
        del self.known_C_corr
        del self.model_string

    def test_gds_correction(self):
        """ Test the computation of the GDS correction """
        calcs = pydarm.calcs.CALCSModel(self.model_string)
        gds_corr = calcs.gds_sensing_correction(self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(gds_corr[n]), np.abs(self.known_gds_correction[n]))
            self.assertAlmostEqual(
                np.angle(gds_corr[n], deg=True),
                np.angle(self.known_gds_correction[n], deg=True))
    
    def test_C_corr(self):
        """ Test the computation of C_corr """
        calcs = pydarm.calcs.CALCSModel(self.model_string)
        C_corr = calcs.C_corr(self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(C_corr[n]), np.abs(self.known_C_corr[n]))
            self.assertAlmostEqual(
                np.angle(C_corr[n], deg=True),
                np.angle(self.known_C_corr[n], deg=True))


class TestDrivealignOutToDisplacement(unittest.TestCase):

    def setUp(self):
        self.frequencies = np.logspace(0, np.log10(5000), 10)
        self.model_string = '''
[metadata]
[interferometer]
[digital]
[sensing]
[actuation]
[calcs]
xarm_uim_analog = test/H1CALCS_ETMX_L1_ANALOG.txt
xarm_pum_analog = test/H1CALCS_ETMX_L2_ANALOG.txt
xarm_tst_analog = test/H1CALCS_ETMX_L3_ANALOG.txt
xarm_output_matrix = 0.0, -1.0, -1.0, -1.0
[actuation_x_arm]
tst_NpV2 = 4.427e-11
actuation_esd_bias_voltage = -9.3
tst_driver_DC_gain_VpV_HV = 40
tst_driver_DC_gain_VpV_LV = 1.881
dac_gain = 7.62939453125e-05
pum_driver_DC_trans_ApV = 2.6847e-4
pum_coil_outf_signflip = 1
pum_NpA = 0.02947
uim_driver_DC_trans_ApV = 6.1535e-4
uim_NpA = 1.634
'''
        self.sign_string = "darm_feedback_sign = -1"
        self.known_darm_uim = np.array(
            [-3.238927901844634e-11+1.4842248249648434e-10j,
             -2.1892938228443525e-13-1.2823546638408861e-13j,
             -8.235508683705688e-16+1.1171018557953846e-17j,
             -2.2721027742100763e-18+3.8079469914466563e-20j,
             -7.727346613339272e-21+2.8219414924360876e-22j,
             -3.07022010410702e-23+2.8033962875878643e-24j,
             -3.7049080828781866e-25+8.515053811571386e-26j,
             -5.6541859151817374e-27+2.7786427108167966e-27j,
             -1.5563418053688328e-31+2.6074253026280297e-32j,
             -9.10020413742017e-36-9.112459957053174e-38j])
        self.known_darm_pum = np.array(
            [-5.478095746328349e-13+1.455441816425798e-12j,
             6.167675927515008e-15+3.476345955497267e-16j,
             8.922276848356977e-17+2.2793664015837933e-20j,
             1.93246038703505e-18+1.0535918169326137e-22j,
             4.400980788166467e-20-6.018841093550202e-24j,
             1.0685506817059288e-21-5.042502461408688e-25j,
             4.0817828174035824e-23-6.304119517788214e-26j,
             -2.1494901791827377e-24+3.4968924666365354e-27j,
             -1.512228887834369e-24+1.3001133772698815e-24j,
             7.827471412730521e-30+6.02486971489429e-33j])
        self.known_darm_tst = np.array(
            [2.0597184624425995e-15+8.489735047270597e-15j,
             4.891873116712453e-16+2.492236617170203e-19j,
             6.918623785211078e-17-8.395825629537092e-21j,
             1.03379064938846e-17-5.105213618025428e-21j,
             1.5555470654579614e-18-2.0854170062766784e-21j,
             2.342445539745114e-19-8.1536988350230305e-22j,
             3.522752328873434e-20-3.1653333930430725e-22j,
             5.241125064242745e-21-1.2206599337631032e-22j,
             7.249945048080412e-22-4.5263355007569324e-23j,
             5.1451821428785154e-23-1.1567627817439817e-23j])
        signval = float(self.sign_string.split(' = ')[-1])
        self.known_long_uim = self.known_darm_uim / signval
        self.known_long_pum = self.known_darm_pum / signval
        self.known_long_tst = self.known_darm_tst / signval

    def tearDown(self):
        del self.frequencies
        del self.model_string
        del self.sign_string
        del self.known_darm_uim
        del self.known_darm_pum
        del self.known_darm_tst
        del self.known_long_uim
        del self.known_long_pum
        del self.known_long_tst

    def test_drivealign_out_to_darm_displacement(self):
        calcs = pydarm.calcs.CALCSModel(self.model_string + self.sign_string)
        test_val = calcs.drivealign_out_to_darm_displacement(
            self.frequencies, arm='x', stage='uim')
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(np.abs(test_val[n]) /
                                   np.abs(self.known_darm_uim[n]), 1)
            self.assertAlmostEqual(np.angle(test_val[n], deg=True),
                                   np.angle(self.known_darm_uim[n], deg=True))
        test_val = calcs.drivealign_out_to_darm_displacement(
            self.frequencies, arm='x', stage='pum')
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(np.abs(test_val[n]) /
                                   np.abs(self.known_darm_pum[n]), 1)
            self.assertAlmostEqual(np.angle(test_val[n], deg=True),
                                   np.angle(self.known_darm_pum[n], deg=True))
        test_val = calcs.drivealign_out_to_darm_displacement(
            self.frequencies, arm='x', stage='tst')
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(np.abs(test_val[n]) /
                                   np.abs(self.known_darm_tst[n]), 1)
            self.assertAlmostEqual(np.angle(test_val[n], deg=True),
                                   np.angle(self.known_darm_tst[n], deg=True))

    def test_drivealign_out_to_longitudinal_displacement(self):
        calcs = pydarm.calcs.CALCSModel(self.model_string)
        test_val = calcs.drivealign_out_to_longitudinal_displacement(
            self.frequencies, arm='x', stage='uim')
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(np.abs(test_val[n]) /
                                   np.abs(self.known_long_uim[n]), 1)
            self.assertAlmostEqual(np.angle(test_val[n], deg=True),
                                   np.angle(self.known_long_uim[n], deg=True))
        test_val = calcs.drivealign_out_to_longitudinal_displacement(
            self.frequencies, arm='x', stage='pum')
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(np.abs(test_val[n]) /
                                   np.abs(self.known_long_pum[n]), 1)
            self.assertAlmostEqual(np.angle(test_val[n], deg=True),
                                   np.angle(self.known_long_pum[n], deg=True))
        test_val = calcs.drivealign_out_to_longitudinal_displacement(
            self.frequencies, arm='x', stage='tst')
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(np.abs(test_val[n]) /
                                   np.abs(self.known_long_tst[n]), 1)
            self.assertAlmostEqual(np.angle(test_val[n], deg=True),
                                   np.angle(self.known_long_tst[n], deg=True))


class TestStageSuperActuator(unittest.TestCase):

    def setUp(self):
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_actuation = np.array(
            [-5.310569188975287e-15+7.256951812950946e-16j,
             2.373277011838796e-15+5.291008654481964e-16j,
             6.565119740746598e-16-1.9982361183467982e-15j,
             -2.2559506195176175e-16-4.466754954217274e-16j,
             -9.777506975890007e-17-5.640474139960699e-17j,
             -1.862001070553775e-17-2.5901317072945293e-18j,
             -2.5387144596970953e-18+2.087430071154024e-19j,
             -3.6579868747198944e-19+8.64886306967107e-20j,
             -3.031250280475011e-20+4.8441976904940323e-20j,
             -3.257386852647821e-21-2.3520741277934774e-21j])

    def tearDown(self):
        del self.frequencies

    def test_stage_super_actuator(self):
        calcs = pydarm.calcs.CALCSModel('''
[metadata]
[interferometer]
[sensing]
[digital]
[actuation]
darm_output_matrix = 1.0, -1.0, 0.0, 0.0
darm_feedback_x = OFF, OFF, OFF, ON
darm_feedback_y = OFF, OFF, OFF, ON
[actuation_x_arm]
darm_feedback_sign = -1
tst_NpV2 = 4.427e-11
linearization = OFF
actuation_esd_bias_voltage = -9.3
tst_driver_DC_gain_VpV_HV = 40
tst_driver_DC_gain_VpV_LV = 1.881
anti_imaging_rate_string = 16k
anti_imaging_method = biquad
analog_anti_imaging_file = test/H1aa.mat
dac_gain = 7.62939453125e-05
sus_filter_file = test/H1SUSETMX_1236641144.txt
tst_isc_inf_bank = ETMX_L3_ISCINF_L
tst_isc_inf_modules =
tst_isc_inf_gain = 1.0
tst_lock_bank = ETMX_L3_LOCK_L
tst_lock_modules = 5,8,9,10
tst_lock_gain = 1.0
tst_drive_align_bank = ETMX_L3_DRIVEALIGN_L2L
tst_drive_align_modules = 4,5
tst_drive_align_gain = -35.7
pum_lock_bank = ETMX_L2_LOCK_L
pum_lock_modules = 7
pum_lock_gain = 23.0
pum_drive_align_bank = ETMX_L2_DRIVEALIGN_L2L
pum_drive_align_modules = 6,7
pum_drive_align_gain = 1.0
uim_lock_bank = ETMX_L1_LOCK_L
uim_lock_modules = 10
uim_lock_gain = 1.06
uim_drive_align_bank = ETMX_L1_DRIVEALIGN_L2L
uim_drive_align_modules =
uim_drive_align_gain = 1.0
[actuation_y_arm]
darm_feedback_sign = 1
tst_NpV2 = 4.427e-11
linearization = OFF
actuation_esd_bias_voltage = -9.3
tst_driver_DC_gain_VpV_HV = 40
tst_driver_DC_gain_VpV_LV = 1.881
anti_imaging_rate_string = 16k
anti_imaging_method = biquad
analog_anti_imaging_file = test/H1aa.mat
dac_gain = 7.62939453125e-05
sus_filter_file = test/H1SUSETMX_1236641144.txt
tst_isc_inf_bank = ETMX_L3_ISCINF_L
tst_isc_inf_modules =
tst_isc_inf_gain = 1.0
tst_lock_bank = ETMX_L3_LOCK_L
tst_lock_modules = 5,8,9,10
tst_lock_gain = 1.0
tst_drive_align_bank = ETMX_L3_DRIVEALIGN_L2L
tst_drive_align_modules = 4,5
tst_drive_align_gain = -35.7
pum_lock_bank = ETMX_L2_LOCK_L
pum_lock_modules = 7
pum_lock_gain = 23.0
pum_drive_align_bank = ETMX_L2_DRIVEALIGN_L2L
pum_drive_align_modules = 6,7
pum_drive_align_gain = 1.0
uim_lock_bank = ETMX_L1_LOCK_L
uim_lock_modules = 10
uim_lock_gain = 1.06
uim_drive_align_bank = ETMX_L1_DRIVEALIGN_L2L
uim_drive_align_modules =
uim_drive_align_gain = 1.0
[calcs]
xarm_uim_analog = test/H1CALCS_ETMX_L1_ANALOG.txt
xarm_pum_analog = test/H1CALCS_ETMX_L2_ANALOG.txt
xarm_tst_analog = test/H1CALCS_ETMX_L3_ANALOG.txt
xarm_output_matrix = 0.0, -1.0, -1.0, -1.0
yarm_output_matrix = 0.0, -1.0, -1.0, -1.0
yarm_uim_analog = test/H1CALCS_ETMX_L1_ANALOG.txt
yarm_pum_analog = test/H1CALCS_ETMX_L2_ANALOG.txt
yarm_tst_analog = test/H1CALCS_ETMX_L3_ANALOG.txt
''')
        test_val = calcs.stage_super_actuator(self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(np.abs(test_val[n]) /
                                   np.abs(self.known_actuation[n]), 1)
            self.assertAlmostEqual(np.angle(test_val[n], deg=True),
                                   np.angle(self.known_actuation[n], deg=True))


class TestGdsActuationCorrection(unittest.TestCase):

    def setUp(self):
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_gds_correction = np.array(
            [0.8404967514499294-0.02251279611738371j,
             0.9907095585751328+0.001962094231205677j,
             0.990763975573271+0.003815548705921684j,
             0.9906221600923891+0.009484886231351873j,
             0.9900442501120555+0.024330364567526422j,
             0.9862620650174786+0.06312091048110094j,
             0.962066130652169+0.17047900283578304j,
             0.8316201623443047+0.5299562916258533j,
             -1.1474171691614994-1.6791062999287774j,
             -1083.636852739504+5122.811137621163j])
        self.known_calcs_tst = np.array(
            [-2.6552845944876435e-15+3.628475906475473e-16j,
             1.186638505919398e-15+2.645504327240982e-16j,
             3.282559870373299e-16-9.991180591733991e-16j,
             -1.1279753097588087e-16-2.233377477108637e-16j,
             -4.8887534879450034e-17-2.8202370699803494e-17j,
             -9.310005352768874e-18-1.2950658536472646e-18j,
             -1.2693572298485476e-18+1.043715035577012e-19j,
             -1.8289934373599472e-19+4.324431534835535e-20j,
             -1.5156251402375056e-20+2.4220988452470162e-20j,
             -1.6286934263239104e-21-1.1760370638967387e-21j])
        self.model_string = '''
[metadata]
[interferometer]
[sensing]
[digital]
[actuation_x_arm]
darm_feedback_sign = -1
tst_NpV2 = 4.427e-11
linearization = OFF
actuation_esd_bias_voltage = -9.3
suspension_file = test/H1susdata_O3.mat
tst_driver_uncompensated_Z_UL = 129.7e3
tst_driver_uncompensated_Z_LL = 90.74e3
tst_driver_uncompensated_Z_UR = 93.52e3
tst_driver_uncompensated_Z_LR = 131.5e3
tst_driver_uncompensated_P_UL = 31.5e3
tst_driver_uncompensated_P_LL = 26.7e3
tst_driver_uncompensated_P_UR = 26.6e3
tst_driver_uncompensated_P_LR = 31.6e3
tst_driver_DC_gain_VpV_HV = 40
tst_driver_DC_gain_VpV_LV = 1.881
anti_imaging_rate_string = 16k
anti_imaging_method = biquad
analog_anti_imaging_file = test/H1aa.mat
dac_gain = 7.62939453125e-05
unknown_actuation_delay = 15e-6
pum_driver_DC_trans_ApV = 2.6847e-4
pum_coil_outf_signflip = 1
pum_NpA = 0.02947
uim_driver_DC_trans_ApV = 6.1535e-4
uim_NpA = 1.634
sus_filter_file = test/H1SUSETMX_1236641144.txt
tst_isc_inf_bank = ETMX_L3_ISCINF_L
tst_isc_inf_modules =
tst_isc_inf_gain = 1.0
tst_lock_bank = ETMX_L3_LOCK_L
tst_lock_modules = 5,8,9,10
tst_lock_gain = 1.0
tst_drive_align_bank = ETMX_L3_DRIVEALIGN_L2L
tst_drive_align_modules = 4,5
tst_drive_align_gain = -35.7
pum_lock_bank = ETMX_L2_LOCK_L
pum_lock_modules = 7
pum_lock_gain = 23.0
pum_drive_align_bank = ETMX_L2_DRIVEALIGN_L2L
pum_drive_align_modules = 6,7
pum_drive_align_gain = 1.0
uim_lock_bank = ETMX_L1_LOCK_L
uim_lock_modules = 10
uim_lock_gain = 1.06
uim_drive_align_bank = ETMX_L1_DRIVEALIGN_L2L
uim_drive_align_modules =
uim_drive_align_gain = 1.0
[actuation]
darm_output_matrix = 1.0, -1.0, 0.0, 0.0
darm_feedback_x = OFF, ON, ON, ON
darm_feedback_y = OFF, OFF, OFF, OFF
[calcs]
xarm_tst_analog = test/H1CALCS_ETMX_L3_ANALOG.txt
xarm_output_matrix = 0.0, 0.0, 0.0, -1.0
'''

    def tearDown(self):
        del self.frequencies
        del self.known_gds_correction
        del self.model_string
        del self.known_calcs_tst

    def test_gds_actuation_correction(self):
        calcs = pydarm.calcs.CALCSModel(self.model_string)
        gds_corr = calcs.gds_actuation_correction(self.frequencies,
                                                  stage='TST')
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(gds_corr[n]) / np.abs(self.known_gds_correction[n]),
                1.0)
            self.assertAlmostEqual(
                np.angle(gds_corr[n], deg=True),
                np.angle(self.known_gds_correction[n], deg=True))

    def test_compute_actuation_single_stage(self):
        calcs = pydarm.calcs.CALCSModel(self.model_string)
        tst = calcs.compute_actuation_single_stage(self.frequencies,
                                                   arm='x',
                                                   stage='TST')
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(tst[n]) / np.abs(self.known_calcs_tst[n]),
                1.0)
            self.assertAlmostEqual(
                np.angle(tst[n], deg=True),
                np.angle(self.known_calcs_tst[n], deg=True))


class TestCalcsDarmActuation(unittest.TestCase):

    def setUp(self):
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.model_string = '''
[metadata]
[interferometer]
[sensing]
[digital]
[actuation]
darm_output_matrix = 1.0, -1.0, 0.0, 0.0
darm_feedback_x = OFF, ON, ON, ON
darm_feedback_y = OFF, OFF, OFF, OFF
[actuation_x_arm]
darm_feedback_sign = -1
tst_NpV2 = 4.427e-11
linearization = OFF
actuation_esd_bias_voltage = -9.3
tst_driver_DC_gain_VpV_HV = 40
tst_driver_DC_gain_VpV_LV = 1.881
anti_imaging_rate_string = 16k
anti_imaging_method = biquad
analog_anti_imaging_file = test/H1aa.mat
dac_gain = 7.62939453125e-05
pum_driver_DC_trans_ApV = 2.6847e-4
pum_coil_outf_signflip = 1
pum_NpA = 0.02947
uim_driver_DC_trans_ApV = 6.1535e-4
uim_NpA = 1.634
sus_filter_file = test/H1SUSETMX_1236641144.txt
tst_isc_inf_bank = ETMX_L3_ISCINF_L
tst_isc_inf_modules =
tst_isc_inf_gain = 1.0
tst_lock_bank = ETMX_L3_LOCK_L
tst_lock_modules = 5,8,9,10
tst_lock_gain = 1.0
tst_drive_align_bank = ETMX_L3_DRIVEALIGN_L2L
tst_drive_align_modules = 4,5
tst_drive_align_gain = -35.7
pum_lock_bank = ETMX_L2_LOCK_L
pum_lock_modules = 7
pum_lock_gain = 23.0
pum_drive_align_bank = ETMX_L2_DRIVEALIGN_L2L
pum_drive_align_modules = 6,7
pum_drive_align_gain = 1.0
uim_lock_bank = ETMX_L1_LOCK_L
uim_lock_modules = 10
uim_lock_gain = 1.06
uim_drive_align_bank = ETMX_L1_DRIVEALIGN_L2L
uim_drive_align_modules =
uim_drive_align_gain = 1.0
[calcs]
xarm_uim_analog = test/H1CALCS_ETMX_L1_ANALOG.txt
xarm_pum_analog = test/H1CALCS_ETMX_L2_ANALOG.txt
xarm_tst_analog = test/H1CALCS_ETMX_L3_ANALOG.txt
xarm_output_matrix = 0.0, -1.0, -1.0, -1.0
'''
        self.known_actuation = np.array(
            [-4.517595087730189e-13-2.574036100879927e-13j,
             -1.588361922503374e-14+6.321091918942617e-15j,
             -2.010750482679724e-15+8.323452747346148e-16j,
             -3.003908484149924e-16+4.0189540926990994e-17j,
             -4.589254777634106e-17+1.7738463389039645e-18j,
             -7.164372681090654e-18-9.924814855271752e-19j,
             -1.2876117069737756e-18+9.59002289638503e-20j,
             -1.8304199780422068e-19+4.3641229060862356e-20j,
             -1.4943625550308623e-20+2.4185781589291237e-20j,
             -1.6286934512050932e-21-1.176037151353447e-21j])

    def tearDown(self):
        del self.frequencies
        del self.model_string
        del self.known_actuation

    def test_calcs_darm_actuation(self):
        calcs = pydarm.calcs.CALCSModel(self.model_string)
        test_actuation = calcs.calcs_darm_actuation(self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs((test_actuation[n]) /
                        np.abs(self.known_actuation[n])), 1)
            self.assertAlmostEqual(
                np.angle(test_actuation[n], deg=True),
                np.angle(self.known_actuation[n], deg=True))

    def test_arm_super_actuator(self):
        calcs = pydarm.calcs.CALCSModel(self.model_string)
        test_actuation = calcs.arm_super_actuator(self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs((test_actuation[n]) /
                        np.abs(self.known_actuation[n])), 1)
            self.assertAlmostEqual(
                np.angle(test_actuation[n], deg=True),
                np.angle(self.known_actuation[n], deg=True))


class TestACorr(unittest.TestCase):

    def setUp(self):
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_correction = np.array(
            [0.8256840102234249+0.040037550804620875j,
             1.0100908537872708-0.03431742186724575j,
             0.9938282915911524+0.0035596494716521737j,
             0.987085789996497+0.007169039492965028j,
             0.9892669716989076+0.02615913560670487j,
             0.9853591419850162+0.06159399166108731j,
             0.9616724418609144+0.17084142887447454j,
             0.8317978218032787+0.529870878348411j,
             -1.152159283662438-1.6878788665759379j,
             -1083.6927058686802+5122.867648656695j])

    def tearDown(self):
        del self.frequencies
        del self.known_correction

    def test_A_corr(self):
        calcs = pydarm.calcs.CALCSModel('''
[metadata]
[interferometer]
[sensing]
[digital]
[actuation]
darm_output_matrix = 1.0, -1.0, 0.0, 0.0
darm_feedback_x = OFF, ON, ON, ON
darm_feedback_y = OFF, OFF, OFF, OFF
[actuation_x_arm]
darm_feedback_sign = -1
tst_NpV2 = 4.427e-11
linearization = OFF
actuation_esd_bias_voltage = -9.3
suspension_file = test/H1susdata_O3.mat
tst_driver_uncompensated_Z_UL = 129.7e3
tst_driver_uncompensated_Z_LL = 90.74e3
tst_driver_uncompensated_Z_UR = 93.52e3
tst_driver_uncompensated_Z_LR = 131.5e3
tst_driver_uncompensated_P_UL = 31.5e3
tst_driver_uncompensated_P_LL = 26.7e3
tst_driver_uncompensated_P_UR = 26.6e3
tst_driver_uncompensated_P_LR = 31.6e3
tst_driver_DC_gain_VpV_HV = 40
tst_driver_DC_gain_VpV_LV = 1.881
anti_imaging_rate_string = 16k
anti_imaging_method = biquad
analog_anti_imaging_file = test/H1aa.mat
dac_gain = 7.62939453125e-05
unknown_actuation_delay = 15e-6
pum_driver_DC_trans_ApV = 2.6847e-4
pum_coil_outf_signflip = 1
pum_NpA = 0.02947
uim_driver_DC_trans_ApV = 6.1535e-4
uim_NpA = 1.634
sus_filter_file = test/H1SUSETMX_1236641144.txt
tst_isc_inf_bank = ETMX_L3_ISCINF_L
tst_isc_inf_modules =
tst_isc_inf_gain = 1.0
tst_lock_bank = ETMX_L3_LOCK_L
tst_lock_modules = 5,8,9,10
tst_lock_gain = 1.0
tst_drive_align_bank = ETMX_L3_DRIVEALIGN_L2L
tst_drive_align_modules = 4,5
tst_drive_align_gain = -35.7
pum_lock_bank = ETMX_L2_LOCK_L
pum_lock_modules = 7
pum_lock_gain = 23.0
pum_drive_align_bank = ETMX_L2_DRIVEALIGN_L2L
pum_drive_align_modules = 6,7
pum_drive_align_gain = 1.0
uim_lock_bank = ETMX_L1_LOCK_L
uim_lock_modules = 10
uim_lock_gain = 1.06
uim_drive_align_bank = ETMX_L1_DRIVEALIGN_L2L
uim_drive_align_modules =
uim_drive_align_gain = 1.0
[calcs]
xarm_uim_analog = test/H1CALCS_ETMX_L1_ANALOG.txt
xarm_pum_analog = test/H1CALCS_ETMX_L2_ANALOG.txt
xarm_tst_analog = test/H1CALCS_ETMX_L3_ANALOG.txt
xarm_output_matrix = 0.0, -1.0, -1.0, -1.0
''')
        test_corr = calcs.A_corr(self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs((test_corr[n]) /
                        np.abs(self.known_correction[n])), 1)
            self.assertAlmostEqual(
                np.angle(test_corr[n], deg=True),
                np.angle(self.known_correction[n], deg=True))


class TestSensingActuationDelay(unittest.TestCase):

    def setUp(self):
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_delay = np.array(
            [-0.04037604748711977,
             -0.004241732151460522,
             0.0008245861775018065,
             0.00042427948404596266,
             0.0004030354338592999,
             0.0004137083970280847,
             0.0004113912830929018,
             0.0004118062637910283,
             -9.851958730979098e-05,
             4.472355469886113e-05])

    def tearDown(self):
        del self.frequencies

    def test_sensing_actuation_delay(self):
        calcs = pydarm.calcs.CALCSModel('''
[metadata]
[interferometer]
[sensing]
x_arm_length = 3994.4704
y_arm_length = 3994.4692
coupled_cavity_optical_gain = 3.22e6
coupled_cavity_pole_frequency = 410.6
detuned_spring_frequency = 4.468
detuned_spring_Q = 52.14
sensing_sign = 1
is_pro_spring = True
anti_aliasing_rate_string = 16k
anti_aliasing_method      = biquad
analog_anti_aliasing_file = test/H1aa.mat, test/H1aa.mat
omc_meas_p_trans_amplifier_uncompensated   = 13.7e3, 17.8e3: 13.7e3, 17.8e3
whitening_mode_names = test, test
omc_meas_p_whitening_uncompensated_test   = 11.346e3, 32.875e3, 32.875e3: 11.521e3, 32.863e3, 32.863e3
gain_ratio = 1, 1
balance_matrix = 1, 1
omc_path_names = A, B
single_pole_approximation_delay_correction = -12e-6
super_high_frequency_poles_apparent_delay = 0, 0
adc_gain = 1638.001638001638, 1638.001638001638
omc_filter_file = test/H1OMC_1239468752.txt
omc_filter_bank = OMC_DCPD_A, OMC_DCPD_B
omc_filter_noncompensating_modules = 4: 4
omc_filter_gain = 1, 1
omc_front_end_trans_amplifier_compensation = ON, ON
omc_front_end_whitening_compensation_test = ON, ON
[digital]
digital_filter_file = test/H1OMC_1239468752.txt
digital_filter_bank = LSC_DARM1, LSC_DARM2
digital_filter_modules = 1,2,3,4,7,9,10: 3,4,5,6,7
digital_filter_gain = 400,1
[actuation]
darm_output_matrix = 1.0, -1.0, 0.0, 0.0
darm_feedback_x = OFF, ON, ON, ON
darm_feedback_y = OFF, OFF, OFF, OFF
[actuation_x_arm]
darm_feedback_sign = -1
tst_NpV2 = 4.427e-11
linearization = OFF
actuation_esd_bias_voltage = -9.3
suspension_file = test/H1susdata_O3.mat
tst_driver_uncompensated_Z_UL = 129.7e3
tst_driver_uncompensated_Z_LL = 90.74e3
tst_driver_uncompensated_Z_UR = 93.52e3
tst_driver_uncompensated_Z_LR = 131.5e3
tst_driver_uncompensated_P_UL = 31.5e3
tst_driver_uncompensated_P_LL = 26.7e3
tst_driver_uncompensated_P_UR = 26.6e3
tst_driver_uncompensated_P_LR = 31.6e3
tst_driver_DC_gain_VpV_HV = 40
tst_driver_DC_gain_VpV_LV = 1.881
anti_imaging_rate_string = 16k
anti_imaging_method = biquad
analog_anti_imaging_file = test/H1aa.mat
dac_gain = 7.62939453125e-05
unknown_actuation_delay = 15e-6
pum_driver_DC_trans_ApV = 2.6847e-4
pum_coil_outf_signflip = 1
pum_NpA = 0.02947
uim_driver_DC_trans_ApV = 6.1535e-4
uim_NpA = 1.634
sus_filter_file = test/H1SUSETMX_1236641144.txt
tst_isc_inf_bank = ETMX_L3_ISCINF_L
tst_isc_inf_modules =
tst_isc_inf_gain = 1.0
tst_lock_bank = ETMX_L3_LOCK_L
tst_lock_modules = 5,8,9,10
tst_lock_gain = 1.0
tst_drive_align_bank = ETMX_L3_DRIVEALIGN_L2L
tst_drive_align_modules = 4,5
tst_drive_align_gain = -35.7
pum_lock_bank = ETMX_L2_LOCK_L
pum_lock_modules = 7
pum_lock_gain = 23.0
pum_drive_align_bank = ETMX_L2_DRIVEALIGN_L2L
pum_drive_align_modules = 6,7
pum_drive_align_gain = 1.0
uim_lock_bank = ETMX_L1_LOCK_L
uim_lock_modules = 10
uim_lock_gain = 1.06
uim_drive_align_bank = ETMX_L1_DRIVEALIGN_L2L
uim_drive_align_modules =
uim_drive_align_gain = 1.0
[calcs]
foton_invsensing_tf = \
  test/2019-04-04_H1CALCS_InverseSensingFunction_Foton_SRCD-2N_Gain_tf.txt
foton_deltal_whitening_tf = \
  test/H1CALCS_DELTAL_EXTERNAL_WHITENING_tf.txt
xarm_uim_analog = test/H1CALCS_ETMX_L1_ANALOG.txt
xarm_pum_analog = test/H1CALCS_ETMX_L2_ANALOG.txt
xarm_tst_analog = test/H1CALCS_ETMX_L3_ANALOG.txt
xarm_output_matrix = 0.0, -1.0, -1.0, -1.0
foton_delay_filter_tf = test/H1CALCS_8_CLK_DELAY.txt
''')
        delay = calcs.sensing_actuation_delay(self.frequencies)
        self.assertTrue(np.allclose(delay/self.known_delay,
                                    np.ones(len(self.frequencies))))


class TestDeltalExtWhitening(unittest.TestCase):

    def setUp(self):
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_tf = np.array(
            [653.22198+1645.6321j,
             -144501.22640332347+382419.13620180736j,
             690389.1200990456+102269154.46218154j,
             14723508812.673355-1068490289.9068925j,
             -293161382327.7433-125492534509.41472j,
             29690171524.466835+816147609224.8961j,
             796229614540.8058+552585560164.8594j,
             968079436179.5526+231562800339.5267j,
             995587610786.119+87377107517.47844j,
             999729543068.4756+23985416987.413254j])

    def tearDown(self):
        del self.frequencies
        del self.known_tf

    def test_deltal_ext_whitening(self):
        calcs = pydarm.calcs.CALCSModel('''
[metadata]
[interferometer]
[sensing]
[digital]
[actuation]
[actuation_x_arm]
[calcs]
foton_deltal_whitening_tf = \
  test/H1CALCS_DELTAL_EXTERNAL_WHITENING_tf.txt
''')
        whitening_tf = calcs.deltal_ext_whitening(self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(whitening_tf[n] / self.known_tf[n]), 1)
            self.assertAlmostEqual(
                np.angle(whitening_tf[n], deg=True),
                np.angle(self.known_tf[n], deg=True))


class TestCalcsDttCalibration(unittest.TestCase):

    def setUp(self):
        # Pre-computed values
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.model_string = '''
[metadata]
[interferometer]
[sensing]
x_arm_length = 3994.4704
y_arm_length = 3994.4692
coupled_cavity_optical_gain = 3.22e6
coupled_cavity_pole_frequency = 410.6
detuned_spring_frequency = 4.468
detuned_spring_Q = 52.14
sensing_sign = 1
is_pro_spring = True
anti_aliasing_rate_string = 16k
anti_aliasing_method      = biquad
analog_anti_aliasing_file = test/H1aa.mat, test/H1aa.mat
omc_meas_p_trans_amplifier_uncompensated   = 13.7e3, 17.8e3: 13.7e3, 17.8e3
whitening_mode_names = test, test
omc_meas_p_whitening_uncompensated_test   = 11.346e3, 32.875e3, 32.875e3: 11.521e3, 32.863e3, 32.863e3
gain_ratio = 1, 1
balance_matrix = 1, 1
omc_path_names = A, B
single_pole_approximation_delay_correction = -12e-6
super_high_frequency_poles_apparent_delay = 0, 0
adc_gain = 1638.001638001638, 1638.001638001638
omc_filter_file = test/H1OMC_1239468752.txt
omc_filter_bank = OMC_DCPD_A, OMC_DCPD_B
omc_filter_noncompensating_modules = 4: 4
omc_filter_gain = 1, 1
omc_front_end_trans_amplifier_compensation = ON, ON
omc_front_end_whitening_compensation_test = ON, ON
[digital]
digital_filter_file = test/H1OMC_1239468752.txt
digital_filter_bank = LSC_DARM1, LSC_DARM2
digital_filter_modules = 1,2,3,4,7,9,10: 3,4,5,6,7
digital_filter_gain = 400,1
[actuation]
darm_output_matrix = 1.0, -1.0, 0.0, 0.0
darm_feedback_x = OFF, ON, ON, ON
darm_feedback_y = OFF, OFF, OFF, OFF
[actuation_x_arm]
darm_feedback_sign = -1
tst_NpV2 = 4.427e-11
linearization = OFF
actuation_esd_bias_voltage = -9.3
suspension_file = test/H1susdata_O3.mat
tst_driver_uncompensated_Z_UL = 129.7e3
tst_driver_uncompensated_Z_LL = 90.74e3
tst_driver_uncompensated_Z_UR = 93.52e3
tst_driver_uncompensated_Z_LR = 131.5e3
tst_driver_uncompensated_P_UL = 31.5e3
tst_driver_uncompensated_P_LL = 26.7e3
tst_driver_uncompensated_P_UR = 26.6e3
tst_driver_uncompensated_P_LR = 31.6e3
tst_driver_DC_gain_VpV_HV = 40
tst_driver_DC_gain_VpV_LV = 1.881
anti_imaging_rate_string = 16k
anti_imaging_method = biquad
analog_anti_imaging_file = test/H1aa.mat
dac_gain = 7.62939453125e-05
unknown_actuation_delay = 15e-6
pum_driver_DC_trans_ApV = 2.6847e-4
pum_coil_outf_signflip = 1
pum_NpA = 0.02947
uim_driver_DC_trans_ApV = 6.1535e-4
uim_NpA = 1.634
sus_filter_file = test/H1SUSETMX_1236641144.txt
tst_isc_inf_bank = ETMX_L3_ISCINF_L
tst_isc_inf_modules =
tst_isc_inf_gain = 1.0
tst_lock_bank = ETMX_L3_LOCK_L
tst_lock_modules = 5,8,9,10
tst_lock_gain = 1.0
tst_drive_align_bank = ETMX_L3_DRIVEALIGN_L2L
tst_drive_align_modules = 4,5
tst_drive_align_gain = -35.7
pum_lock_bank = ETMX_L2_LOCK_L
pum_lock_modules = 7
pum_lock_gain = 23.0
pum_drive_align_bank = ETMX_L2_DRIVEALIGN_L2L
pum_drive_align_modules = 6,7
pum_drive_align_gain = 1.0
uim_lock_bank = ETMX_L1_LOCK_L
uim_lock_modules = 10
uim_lock_gain = 1.06
uim_drive_align_bank = ETMX_L1_DRIVEALIGN_L2L
uim_drive_align_modules =
uim_drive_align_gain = 1.0
[pcal]
pcal_dewhiten = 1.0, 1.0
ref_pcal_2_darm_act_sign = -1
analog_anti_aliasing_file = test/H1aa.mat
anti_aliasing_rate_string = 16k
anti_aliasing_method = biquad
[calcs]
foton_invsensing_tf = \
  test/2019-04-04_H1CALCS_InverseSensingFunction_Foton_SRCD-2N_Gain_tf.txt
foton_deltal_whitening_tf = \
  test/H1CALCS_DELTAL_EXTERNAL_WHITENING_tf.txt
xarm_uim_analog = test/H1CALCS_ETMX_L1_ANALOG.txt
xarm_pum_analog = test/H1CALCS_ETMX_L2_ANALOG.txt
xarm_tst_analog = test/H1CALCS_ETMX_L3_ANALOG.txt
xarm_output_matrix = 0.0, -1.0, -1.0, -1.0
foton_delay_filter_tf = test/H1CALCS_8_CLK_DELAY.txt
'''
        self.known_dtt_calibration = np.array(
            [0.00019356005813191712-0.00042488196251998944j,
             -9.451078100284736e-07-2.284431871622057e-06j,
             1.7790453057980935e-10-9.716717778553878e-09j,
             6.612312986720728e-11+6.670843982848847e-12j,
             -3.0625580407492805e-12+9.707481479785013e-13j,
             1.7153848960220716e-13-1.254844015881104e-12j,
             9.785864383613003e-13-2.354168502599546e-13j,
             7.738102950993615e-13+6.274479982018254e-13j,
             -6.479370603503953e-13+7.062109756021947e-13j,
             8.255633601165232e-13+3.1898890750452897e-13j])
        self.known_correction = np.array(
            [-0.0008501089862348232-0.0003863633736571571j,
             -1.7116997942422424e-05-7.969078324489177e-06j,
             -1.238116795723849e-07-4.199824052455882e-07j,
             1.9489520393474323e-08-6.143763314670491e-10j,
             -5.7662915446536716e-09+2.38094579579211e-09j,
             2.7970665627508423e-10-1.6315937801218042e-08j,
             7.545157306911198e-08-4.156187896590543e-08j,
             5.689969540529809e-07+3.095147054562399e-09j,
             3.203791528674027e-06+1.940085939941273e-06j,
            -1.7077191837064583e-06+2.1903105417274858e-05j])

    def tearDown(self):
        del self.frequencies
        del self.model_string
        del self.known_dtt_calibration
        del self.known_correction

    def test_calcs_dtt_calibration(self):
        calcs = pydarm.calcs.CALCSModel(self.model_string)
        calcs_dtt_calib = calcs.calcs_dtt_calibration(self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(calcs_dtt_calib[n] /
                       np.abs(self.known_dtt_calibration[n])), 1)
            self.assertAlmostEqual(
                np.angle(calcs_dtt_calib[n], deg=True),
                np.angle(self.known_dtt_calibration[n], deg=True))

    def test_deltal_ext_pcal_corrrection(self):
        calcs = pydarm.calcs.CALCSModel(self.model_string)
        deltal_ext_pcal = calcs.deltal_ext_pcal_correction(self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(deltal_ext_pcal[n] /
                       np.abs(self.known_correction[n])), 1)
            self.assertAlmostEqual(
                np.angle(deltal_ext_pcal[n], deg=True),
                np.angle(self.known_correction[n], deg=True))


class TestComputeEpicsRecords(unittest.TestCase):

    known_epics = {
            'CAL-CS_TDEP_PCAL_LINE1_CORRECTION_REAL': 0.003384106838174459,
            'CAL-CS_TDEP_PCAL_LINE1_CORRECTION_IMAG': 0.0004044180029064908,
            'CAL-CS_TDEP_SUS_LINE3_REF_INVA_TST_RESPRATIO_REAL': 1196925220371225.5,
            'CAL-CS_TDEP_SUS_LINE3_REF_INVA_TST_RESPRATIO_IMAG': -3841998698753872,
            'CAL-CS_TDEP_SUS_LINE2_REF_INVA_PUM_RESPRATIO_REAL': -4.90204412692278e+17,
            'CAL-CS_TDEP_SUS_LINE2_REF_INVA_PUM_RESPRATIO_IMAG': -1.9987201437993693e+17,
            'CAL-CS_TDEP_SUS_LINE1_REF_INVA_UIM_RESPRATIO_REAL': 2.643473241664109e+17,
            'CAL-CS_TDEP_SUS_LINE1_REF_INVA_UIM_RESPRATIO_IMAG': -1.2695297228124422e+17,
            'CAL-CS_TDEP_PCAL_LINE2_REF_C_NOCAVPOLE_REAL': 3052033.7355621597,
            'CAL-CS_TDEP_PCAL_LINE2_REF_C_NOCAVPOLE_IMAG': -1030751.4694586383,
            'CAL-CS_TDEP_PCAL_LINE2_REF_D_REAL': -243010006.4310854,
            'CAL-CS_TDEP_PCAL_LINE2_REF_D_IMAG': -1207741923.4087756,
            'CAL-CS_TDEP_PCAL_LINE2_REF_A_TST_REAL': -3.6301159301930236e-19,
            'CAL-CS_TDEP_PCAL_LINE2_REF_A_TST_IMAG': 5.17252131286287e-19,
            'CAL-CS_TDEP_PCAL_LINE2_REF_A_PUM_REAL': -3.486603813748813e-21,
            'CAL-CS_TDEP_PCAL_LINE2_REF_A_PUM_IMAG': 2.854144501791321e-21,
            'CAL-CS_TDEP_PCAL_LINE2_REF_A_UIM_REAL': 9.669640696236252e-23,
            'CAL-CS_TDEP_PCAL_LINE2_REF_A_UIM_IMAG': 1.0084874176645285e-22,
            'CAL-CS_TDEP_PCAL_LINE2_CORRECTION_REAL': 5.919275925544666e-06,
            'CAL-CS_TDEP_PCAL_LINE2_CORRECTION_IMAG': +3.317567316630787e-07,
            'CAL-CS_TDEP_PCAL_LINE1_REF_C_NOCAVPOLE_REAL': 3219708.6791652623,
            'CAL-CS_TDEP_PCAL_LINE1_REF_C_NOCAVPOLE_IMAG': -43662.16666949552,
            'CAL-CS_TDEP_PCAL_LINE1_REF_D_REAL': 3152795532.787565,
            'CAL-CS_TDEP_PCAL_LINE1_REF_D_IMAG': 60933868.87924543,
            'CAL-CS_TDEP_PCAL_LINE1_REF_A_TST_REAL': -1.1944749080760831e-16,
            'CAL-CS_TDEP_PCAL_LINE1_REF_A_TST_IMAG': -2.1720205784065532e-16,
            'CAL-CS_TDEP_PCAL_LINE1_REF_A_PUM_REAL': -2.1758869487866404e-16,
            'CAL-CS_TDEP_PCAL_LINE1_REF_A_PUM_IMAG': 2.620157641796169e-16,
            'CAL-CS_TDEP_PCAL_LINE1_REF_A_UIM_REAL': 4.078008711853952e-17,
            'CAL-CS_TDEP_PCAL_LINE1_REF_A_UIM_IMAG': 4.992745126138772e-18,
            'CAL-CS_TDEP_PCAL_LINE3_CORRECTION_REAL': 8.319363469674813e-07,
            'CAL-CS_TDEP_PCAL_LINE3_CORRECTION_IMAG': 1.1949591405958398e-07,
            'CAL-CS_TDEP_PCAL_X_COMPARE_CORRECTION_REAL': -3.978067573573798e-06,
            'CAL-CS_TDEP_PCAL_X_COMPARE_CORRECTION_IMAG': -2.6500091829175134e-07,
            'CAL-CS_TDEP_PCAL_Y_COMPARE_CORRECTION_REAL': 3.978067573573798e-06,
            'CAL-CS_TDEP_PCAL_Y_COMPARE_CORRECTION_IMAG': 2.6500091829175134e-07,
            'CAL-CS_TDEP_SUS_LINE1_SUS_DEMOD_PHASE': -0.3427734375,
            'CAL-CS_TDEP_SUS_LINE2_SUS_DEMOD_PHASE': -0.36035156249999994,
            'CAL-CS_TDEP_SUS_LINE3_SUS_DEMOD_PHASE': -0.38671875}

    model_string = '''
[metadata]
[interferometer]
[sensing]
x_arm_length = 3994.4704
y_arm_length = 3994.4692
coupled_cavity_optical_gain = 3.22e6
coupled_cavity_pole_frequency = 410.6
detuned_spring_frequency = 4.468
detuned_spring_Q = 52.14
sensing_sign = 1
is_pro_spring = True
anti_aliasing_rate_string = 16k
anti_aliasing_method      = biquad
analog_anti_aliasing_file = test/H1aa.mat, test/H1aa.mat
omc_meas_p_trans_amplifier_uncompensated   = 13.7e3, 17.8e3: 13.7e3, 17.8e3
whitening_mode_names = test, test
omc_meas_p_whitening_uncompensated_test   = 11.346e3, 32.875e3, 32.875e3: 11.521e3, 32.863e3, 32.863e3
super_high_frequency_poles_apparent_delay = 0, 0
gain_ratio = 1, 1
balance_matrix = 1, 1
omc_path_names = A, B
single_pole_approximation_delay_correction = -12e-6
adc_gain = 1, 1
omc_front_end_trans_amplifier_compensation = ON, ON
omc_front_end_whitening_compensation_test = ON, ON

[actuation_x_arm]
darm_feedback_sign = -1
tst_NpV2 = 4.427e-11
linearization = OFF
actuation_esd_bias_voltage = -9.3
suspension_file = test/H1susdata_O3.mat
tst_driver_uncompensated_Z_UL = 129.7e3
tst_driver_uncompensated_Z_LL = 90.74e3
tst_driver_uncompensated_Z_UR = 93.52e3
tst_driver_uncompensated_Z_LR = 131.5e3
tst_driver_uncompensated_P_UL = 3.213e3, 31.5e3
tst_driver_uncompensated_P_LL = 3.177e3, 26.7e3
tst_driver_uncompensated_P_UR = 3.279e3, 26.6e3
tst_driver_uncompensated_P_LR = 3.238e3, 31.6e3
tst_driver_DC_gain_VpV_HV = 40
tst_driver_DC_gain_VpV_LV = 1.881
anti_imaging_rate_string = 16k
anti_imaging_method = biquad
analog_anti_imaging_file = test/H1aa.mat
dac_gain = 7.62939453125e-05
unknown_actuation_delay = 15e-6
pum_driver_DC_trans_ApV = 2.6847e-4
pum_coil_outf_signflip = 1
pum_NpA = 0.02947
uim_driver_DC_trans_ApV = 6.1535e-4
uim_NpA = 1.634
sus_filter_file = test/H1SUSETMX_1236641144.txt
tst_isc_inf_bank = ETMX_L3_ISCINF_L
tst_isc_inf_modules =
tst_isc_inf_gain = 1.0
tst_lock_bank = ETMX_L3_LOCK_L
tst_lock_modules = 5,8,9,10
tst_lock_gain = 1.0
tst_drive_align_bank = ETMX_L3_DRIVEALIGN_L2L
tst_drive_align_modules = 4,5
tst_drive_align_gain = -35.7
pum_lock_bank = ETMX_L2_LOCK_L
pum_lock_modules = 7
pum_lock_gain = 23.0
pum_drive_align_bank = ETMX_L2_DRIVEALIGN_L2L
pum_drive_align_modules = 6,7
pum_drive_align_gain = 1.0
uim_lock_bank = ETMX_L1_LOCK_L
uim_lock_modules = 10
uim_lock_gain = 1.06
uim_drive_align_bank = ETMX_L1_DRIVEALIGN_L2L
uim_drive_align_modules =
uim_drive_align_gain = 1.0

[actuation]
darm_output_matrix = 1.0, -1.0, 0.0, 0.0
darm_feedback_x = OFF, ON, ON, ON
darm_feedback_y = OFF, OFF, OFF, OFF

[pcal]
pcal_dewhiten = 1.0, 1.0
ref_pcal_2_darm_act_sign = -1
analog_anti_aliasing_file = test/H1aa.mat
anti_aliasing_rate_string = 16k
anti_aliasing_method = biquad

[digital]
digital_filter_file = test/H1OMC_1239468752.txt
digital_filter_bank = LSC_DARM1, LSC_DARM2
digital_filter_modules = 1,2,3,4,7,9,10: 3,4,5,6,7
digital_filter_gain = 400,1

[calcs]
cal_line_sus_uim_frequency = 15.6
cal_line_sus_pum_frequency = 16.4
cal_line_sus_tst_frequency = 17.6
cal_line_sus_pcal_frequency = 17.1
cal_line_sens_pcal_frequency = 410.3
cal_line_high_pcal_frequency = 1083.7
cal_line_cmp_pcalx_frequency = 500.1
cal_line_cmp_pcaly_frequency = 500.1
'''

    def test_compute_epics_records(self):
        calcs = pydarm.calcs.CALCSModel(self.model_string)
        test_epics = calcs.compute_epics_records(endstation=True)
        for key, val in self.known_epics.items():
            # FIXME: is this enough precision?
            self.assertAlmostEqual(test_epics[key]/val, 1.0, places=6)

    def test_epics_write_to_file(self):
        calcs = pydarm.calcs.CALCSModel(self.model_string)
        test_epics = calcs.compute_epics_records(endstation=True)
        # FIXME: if we were using pytetst we could simply provide a
        # `tmp_file` argument to the test method to get a temporary
        # file from the test harness
        with tempfile.TemporaryDirectory() as d:
            save_file = os.path.join(d, 'test_epics_out')
            pydarm.utils.write_dict_epics(test_epics, dry_run=True, save_file=save_file)
            with open(save_file) as f:
                for line in f.readlines():
                    chan, val = line.strip().split()
                    self.assertAlmostEqual(self.known_epics[chan]/float(val), 1.0, places=6)

if __name__ == '__main__':
    unittest.main()



# Copyright (C) Ethan Payne (2020)
#               Evan Goetz (2021)
#
# This file is part of pyDARM.
#
# pyDARM is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# pyDARM is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# pyDARM. If not, see <https://www.gnu.org/licenses/>.

import os
import configparser

import numpy as np
from scipy.signal import dfreqresp

from .analog import analog_aa_or_ai_filter_response
from .digital import iopdownsamplingfilters, daqdownsamplingfilters


# set up basic functions to check the type
def isfloat(value):
    try:
        float(value)
        return True
    except ValueError:
        return False


class Model(object):
    """pyDARM Model class

    Represents a transfer function.

    """
    def __init__(self, config, measurement=None):
        """Initialize Model object

        Parameters
        ----------
        config : file path or string
            INI config
        measurement : string
            measurement type, corresponding to section in the config INI

        """
        self._config = None
        self.measurement = measurement
        self._load_configuration(config)

    def _load_configuration(self, config):
        """Reads configuration and load parameters

        Config can be either file path or configuration string.
        Config interface is stored in `self._config` attribute.

        Parameters
        ----------
        config : file path or string
            INI config

        """
        if self._config is None:
            self._config = configparser.ConfigParser(
                comment_prefixes=('#',), inline_comment_prefixes=('#',))

        try:
            if type(config) is dict:
                self._config.read_dict(config)
            elif os.path.exists(os.path.normpath(config)):
                with open(os.path.normpath(config)) as f:
                    self._config.read_file(f)
            else:
                self._config.read_string(config)
        except Exception:
            raise ValueError("Config File is not found:", config)

        if self.measurement:
            for key, value in self._config[self.measurement].items():
                self._set_attribute(key, value)

        if 'metadata' in self._config:
            for key, value in self._config['metadata'].items():
                self._set_attribute(key, value)

        if 'interferometer' in self._config:
            for key, value in self._config['interferometer'].items():
                self._set_attribute(key, value)

    def _set_attribute(self, key, value):
        """Set Model attribute from config key/value

        """
        # Special case for module arrays
        # Sometimes (not always) keys that have the string '_modules' in their
        # name need to have a list of lists. To indicate this we use ':' in
        # the value to represent the separation of these lists
        # So, for _modules, the possibilities are:
        # - a list of 1 empty list
        # - a list of lists, where ':' separates each list
        # - a list of 1 list with values and no ':' was given in the value
        if '_modules' in key:
            if len(value) == 0:
                array = [[]]
            elif ':' in value:
                array = value.split(':')
                for index, inner_array in enumerate(array):
                    if len(inner_array) < 1:
                        inner_array = []
                    else:
                        inner_array = [
                            int(arr_entry)
                            for arr_entry in inner_array.split(',')]
                    array[index] = inner_array
            else:
                array = [[int(value_entry) for value_entry in value.split(',')]]

            value = array

        # Special case for compact measured zeros and poles of the OMC paths
        # This is like the module list of lists except instead of integers, we
        # want floats
        # - OMC paths measured zeros and poles
        elif 'omc_meas_' in key:
            if len(value) == 0:
                array = [[]]
            elif ':' in value:
                array = value.split(':')
                for index, inner_array in enumerate(array):
                    inner_array = [float(arr_entry) for arr_entry in
                                   inner_array.split(',')]
                    array[index] = inner_array
            else:
                array = [[float(value_entry) for value_entry in
                          value.split(',')]]

            value = array

        # Special case for arrays of strings that are separated by ','
        # - OMC path names
        # - OMC path analog whitening mode names
        # - sensing/pcal function anti-aliasing file names
        # - sensing/pcal function anti-aliasing rate string
        # - sensing/pcal function anti-aliasing method
        # - arm actuation function anti-imaging rate string
        # - arm actuation function anti-imaging method
        # - OMC compensation filter bank
        # - digital filter bank names
        # - OMC path compensation active
        elif (key == 'omc_path_names' or
                key == 'whitening_mode_names' or
                (self.measurement in ['sensing', 'pcal'] and
                 (key == 'analog_anti_aliasing_file')) or
                (self.measurement in ['sensing', 'actuation_x_arm', 'actuation_y_arm', 'pcal'] and
                 ('ing_rate_string' in key or 'ing_method' in key)) or
                ('omc_' in key and '_bank' in key) or
                key == 'digital_filter_bank' or
                (self.measurement == 'sensing' and
                 'compensation' in key)):
            value = value.strip(',')
            array = [str(value_entry).strip() for value_entry in
                     value.split(',')]

            value = array

        # Special case if the _meas_z or _meas_p values are empty
        # Aside from the OMC paths measured zeros and poles, if any other
        # measured zeros and poles are empty, then assign an empty list
        # because we don't want an empty string.
        # By using `'compensated...' in key`, this covers both "compensated"
        # and "uncompensated" keys
        elif (('compensated_z' in key and value == '') or
              ('compensated_p' in key and value == '')):
            value = []

        # Special case for the (un)compensated zeros and poles in actuation
        # These need to be in a list of floats separated by commas (,)
        elif ((self.measurement == 'actuation_x_arm' or self.measurement == 'actuation_y_arm') and
              ('compensated_z' in key or 'compensated_p' in key)):
            value.strip(',')
            array = [float(value_entry) for value_entry in value.split(',')]
            value = array

        # Special case for array of gain values (floats) separated by ','
        # where the gains need to be in a list, even if just one element
        # - sensing OMC path digital gains
        # - sensing OMC path gain ratios
        # - sensing balance matrix values
        # - sensing ADC gain values
        # - digital filter bank gain
        # - sensing OMC analog electronics apparent delay from high freq poles
        elif (('omc_' in key and '_gain' in key) or
              'gain_ratio' in key or 'balance_matrix' in key or
              key == 'adc_gain' or key == 'digital_filter_gain' or
              key == 'super_high_frequency_poles_apparent_delay'):
            value = [float(value_entry) for value_entry in value.split(',')]

        # Special case for True or False as values, convert string to python
        # bool
        elif 'True' in value:
            value = True
        elif 'False' in value:
            value = False

        # Check if the value is a float
        elif isfloat(value):
            value = float(value)

        # check if the configuration entry is a 2D array
        # this is more basic than the special cases above
        elif ':' in value:
            array = value.split(':')
            for index, inner_array in enumerate(array):
                inner_array = \
                    [float(arr_entry) for arr_entry in inner_array.split(',')]
                array[index] = inner_array

            value = array

        # Check if it is a 1D array
        # this is more basic than the special cases above
        elif ',' in value:
            value = value.strip(',')
            # Check if X or Y is in the string or make floats or integers
            if ('ON' in value or 'OFF' in value or 'DARM' in value):
                array = [str(value_entry).strip()
                         for value_entry in value.split(',')]
            elif '.' in value:
                array = [float(value_entry)
                         for value_entry in value.split(',')]
            else:
                array = [int(value_entry) for value_entry in value.split(',')]

            value = array

        setattr(self, key, value)

    def dpath(self, *args):
        """Return path to data file

        Path should be relative to the directory specified in the
        `cal_data_root` configuration variable, which may be
        overridden with the CAL_DATA_ROOT environment variable.  If
        not specified, paths will be assumed to be relative to the
        current working directory.

        """
        root = os.getenv('CAL_DATA_ROOT', getattr(self, 'cal_data_root', ''))
        return os.path.join(root, *args)

    def config_to_dict(self):
        """
        Return a nested dict of the model configuration. Sections are dict
        in of themselves

        Returns
        -------
        out : dict
            dictionary of the model parameters

        """
        out = {}

        # loop over each of the sections
        for idx, val in enumerate(self._config.sections()):
            this_sect = {}

            # loop over each item in the section
            for idx2, val2 in enumerate(self._config.items(f'{val}')):
                this_sect[f'{val2[0]}'] = f'{val2[1]}'

            # add this section to the output
            out[f'{val}'] = this_sect

        return out

    def analog_aa_or_ai_filter_response(self, frequencies, idx=0):
        """
        Compute the analog anti-aliasing or anti-imaging filter response

        Parameters
        ----------
        frequencies : `float`, array-like
            array of frequencies to compute the response
        idx : `int`, optional
            if multiple files provided, use an index like 0, 1, or 2 to access
            the appropriate file (default = 0)

        Returns
        -------
        tf : `complex128`, array-like
            transfer function response of the analog AA or AI filter

        """
        path = ''
        if hasattr(self, 'analog_anti_aliasing_file'):
            path = self.analog_anti_aliasing_file[idx]
        elif hasattr(self, 'analog_anti_imaging_file'):
            path = self.analog_anti_imaging_file

        # in case the file is an empty string, we set the response to ones
        # otherwise we actually compute this
        if path == '':
            response = np.ones(len(frequencies), dtype='complex128')
        else:
            response = analog_aa_or_ai_filter_response(self.dpath(path),
                                                       frequencies)

        return response

    def digital_aa_or_ai_filter_response(self, frequencies):
        """
        Compute the digital anti-aliasing or -imaging filter response

        Parameters
        ----------
        frequencies : `float`, array-like
            array of frequencies to compute the response

        Returns
        -------
        tf : `complex128`, array-like
            transfer function response of the digital filter

        """

        # anti_aliasing_rate_string could be multiple strings
        if hasattr(self, 'anti_aliasing_rate_string'):
            rate_string = self.anti_aliasing_rate_string
            method = self.anti_aliasing_method
        else:
            rate_string = self.anti_imaging_rate_string
            method = self.anti_imaging_method

        response = np.ones(len(frequencies), dtype='complex128')

        for idx, this_rate_str in enumerate(rate_string):
            if this_rate_str != '':
                # Here we use a crappy 8x DAQ downsampling filter for the 512k
                # DAQ filter
                if this_rate_str == '512k-daq':
                    filt_ss = daqdownsamplingfilters(
                        524288, 65536, method[idx], rcg_ver=3)
                else:
                    filt_ss = iopdownsamplingfilters(
                        this_rate_str, method[idx], rcg_ver=3)

                filt_zpk = filt_ss.to_zpk()

                # The 512k rate needs to be specified seprately from the 65k
                if this_rate_str == '512k-daq':
                    response *= dfreqresp(
                        filt_zpk, 2.0*np.pi*frequencies/2**19)[1]
                else:
                    response *= dfreqresp(
                        filt_zpk, 2.0*np.pi*frequencies/2**16)[1]

        return response



import os


# instrument designator
IFO = os.getenv('IFO')


# calibration config/data roots
CAL_CONFIG_ROOT = None
CAL_DATA_ROOT = None
if IFO:
    CAL_CONFIG_ROOT = os.path.join(
        '/ligo/groups/cal/ifo/',
        IFO,
    )
    CAL_DATA_ROOT = os.path.join(
        '/ligo/groups/cal/data/',
        IFO,
    )
CAL_CONFIG_ROOT = os.getenv(
    'CAL_CONFIG_ROOT',
    CAL_CONFIG_ROOT,
)
CAL_DATA_ROOT = os.getenv(
    'CAL_DATA_ROOT',
    CAL_DATA_ROOT,
)


CAL_TEMPLATE_ROOT = None
CAL_MEASUREMENT_ROOT = None
DEFAULT_MODEL_PATH = None
DEFAULT_CONFIG_PATH = None
if CAL_CONFIG_ROOT:
    CAL_TEMPLATE_ROOT = os.path.join(
        CAL_CONFIG_ROOT,
        'templates',
    )
    CAL_MEASUREMENT_ROOT = os.path.join(
        CAL_DATA_ROOT,
        'measurements',
    )
    if IFO:
        DEFAULT_MODEL_PATH = os.path.join(
            CAL_CONFIG_ROOT,
            f'pydarm_{IFO}.ini',
        )
        DEFAULT_CONFIG_PATH = os.path.join(
            CAL_CONFIG_ROOT,
            f'pydarm_cmd_{IFO}.yaml',
        )


# default frequency response frequency array
DEFAULT_FREQSPEC = '0.01:5000:3000'



import unittest
import pydarm
import numpy as np


class TestAnalogAAFilterResponse(unittest.TestCase):

    def setUp(self):
        # Pre-computed values
        # This uses the H1 AA response that has a 0.99 gain
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_analog_aa_filter_response = np.array(
            [0.9902583151820756-0.00023625869326662408j,
             0.9902581352102275-0.0006086735508483027j,
             0.990256940678447-0.001568125690903292j,
             0.9902490122026717-0.00403995033942778j,
             0.9901963893790059-0.010407889389582571j,
             0.9898471618373895-0.02680975201205813j,
             0.9875313105265322-0.06899983685404412j,
             0.9722512045409587-0.17657342104176357j,
             0.8745742785659897-0.43547246366239517j,
             0.3390985734013402-0.8563173043598763j])

    def tearDown(self):
        del self.frequencies
        del self.known_analog_aa_filter_response

    def test_analog_aa_filter_response(self):
        """ Test the analog AA filter response """
        pcal = pydarm.pcal.PcalModel('''
[metadata]
[interferometer]
[pcal]
analog_anti_aliasing_file = test/H1aa.mat
''')
        analog_aa_response = pcal.analog_aa_or_ai_filter_response(
            self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(analog_aa_response[n]),
                np.abs(self.known_analog_aa_filter_response[n]))
            self.assertAlmostEqual(
                np.angle(analog_aa_response[n], deg=True),
                np.angle(self.known_analog_aa_filter_response[n], deg=True))


class TestAnalogAIFilterResponse(unittest.TestCase):

    def setUp(self):
        # Pre-computed values
        # This uses the H1 AI response that has a 0.99 gain
        self.frequencies = np.logspace(0, np.log10(5000.), 10)
        self.known_analog_ai_filter_response = np.array(
            [0.9902583151820756-0.00023625869326662408j,
             0.9902581352102275-0.0006086735508483027j,
             0.990256940678447-0.001568125690903292j,
             0.9902490122026717-0.00403995033942778j,
             0.9901963893790059-0.010407889389582571j,
             0.9898471618373895-0.02680975201205813j,
             0.9875313105265322-0.06899983685404412j,
             0.9722512045409587-0.17657342104176357j,
             0.8745742785659897-0.43547246366239517j,
             0.3390985734013402-0.8563173043598763j])

    def tearDown(self):
        del self.frequencies
        del self.known_analog_ai_filter_response

    def test_analog_aa_filter_response(self):
        """ Test the analog AI filter response """
        A = pydarm.actuation.ActuationModel('''
[metadata]
[interferometer]
[actuation_x_arm]
analog_anti_imaging_file = test/H1aa.mat
''',
                                            measurement='actuation_x_arm')
        analog_ai_response = A.analog_aa_or_ai_filter_response(
            self.frequencies)
        for n in range(len(self.frequencies)):
            self.assertAlmostEqual(
                np.abs(analog_ai_response[n]),
                np.abs(self.known_analog_ai_filter_response[n]))
            self.assertAlmostEqual(
                np.angle(analog_ai_response[n], deg=True),
                np.angle(self.known_analog_ai_filter_response[n], deg=True))


if __name__ == '__main__':
    unittest.main()



import numpy as np
from pydarm import measurement
from matplotlib import pyplot as plt

"""
Example code on how to use the following classes from sensing.py:
    - ProcessSensingMeasurement
"""

# Note that you may need to specify where the aligocalibration SVN repo
# base path is on your local checkout. For example, you may have it checked
# out to your home directory instead of /ligo/svncommon/ so in the example
# configuration file, you would change
#   cal_data_root = /ligo/svncommon/aligocalibration/trunk
# to
#   cal_data_root = /home/albert.einstein/aligocalibration/trunk
# This can also be overridden with the CAL_DATA_ROOT environment variable.

# These are the xml files we want to get our data from.
measurement_file_1 = \
    '../test/2020-01-03_H1_DARM_OLGTF_LF_SS_5to1100Hz_15min.xml'
measurement_file_2 = \
    '../test/2020-01-03_H1_PCALY2DARMTF_LF_SS_5t1100Hz_10min.xml'

# This is the model file that we need to use in order to process the data.
model_file = '../example_model_files/H1_20190416.ini'

# ----------------------------------------------------------------------
# If you want to get the processed sensing data for input to the  MCMC
# process, do the following:
meas1 = measurement.Measurement(measurement_file_1)
meas2 = measurement.Measurement(measurement_file_2)

process_sensing = measurement.ProcessSensingMeasurement(
    model_file, meas1, meas2,
    ('H1:LSC-DARM1_IN2', 'H1:LSC-DARM1_EXC'),
    ('H1:CAL-PCALY_RX_PD_OUT_DQ', 'H1:LSC-DARM_IN1_DQ'),
    0.9, 0.9999)

# The method below returns 3 arrays:
# frequency, corrected sensing, relative uncertainty.
# Here is where you tell the ProcessMeasurement object what channels
# are used. The list (channel A, channel B) will extract the transfer
# function channel B / channel A from the DTT file. The first of the two
# transfer functions should be the closed loop gain, and the second should
# be the PCAL to DARM transfer function
frequencies, processed_optical_response, processed_optical_response_unc = \
    process_sensing.get_processed_measurement_response()

# Now we can plot the data.

plt.figure(1)
plt.subplot(211)
plt.loglog(frequencies, np.abs(processed_optical_response), 'ro')
plt.subplot(212)
plt.semilogx(frequencies, np.angle(processed_optical_response, deg=True), 'ro')
plt.show(block=True)
